@@ -1,5 +1,5 @@
 /*
- * "$Id: ipp-support.c 12995 2015-11-20 02:32:42Z msweet $"
+ * "$Id: ipp-support.c 12666 2015-05-25 19:38:09Z msweet $"
  *
  * Internet Printing Protocol support functions for CUPS.
  *
@@ -78,8 +78,7 @@ static const char * const ipp_status_oks[] =	/* "OK" status codes */
 		  "client-error-account-info-needed",
 		  "client-error-account-closed",
 		  "client-error-account-limit-reached",
-		  "client-error-account-authorization-failed",
-		  "client-error-not-fetchable"
+		  "client-error-account-authorization-failed"
 		},
 		* const ipp_status_480s[] =	/* Vendor client errors */
 		{
@@ -2286,5 +2285,5 @@ ipp_col_string(ipp_t  *col,		/* I - Collection attribute */
 
 
 /*
- * End of "$Id: ipp-support.c 12995 2015-11-20 02:32:42Z msweet $".
+ * End of "$Id: ipp-support.c 12666 2015-05-25 19:38:09Z msweet $".
  */
